/*
  This file is part of CinLife

  CinLife is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  CinLife is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with CinLife.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CINLIFE_H_
#define CINLIFE_H_

#include <stdbool.h>
#include <getopt.h>

/* Dimension of the virtual topology */
#define TOPOLOGY_DIMENSION 2

typedef struct local_life_t {
  int      size; /* Size of MPI_COMM_WORLD */
  int      rank; /* Rank in MPI_COMM_WORLD */
  //MPI_Comm cartesian_communicator; /* Cartesian communicator */
  int      cartesian_communicator_length[TOPOLOGY_DIMENSION]; /* Length of each dimension of the virtual topology */
  int      cartesian_communicator_size; /* Size of the cartesian communicator */
  int      cartesian_communicator_rank; /* Rank in the cartesian communicator */
  int      cartesian_coords[TOPOLOGY_DIMENSION]; /* Coordinates in the cartesian communicator */
  int      gsizex,gsizey; /* X and Y size of the global domain */
  char *   global_domain; /* Pointer to global domain memory area (only rank == 0 allocate it, could be optimized away) */
  int      nrows ,ncols;  /* X and Y size of the local domain */
  char *   local_domain;  /* Pointer to the local domain memory area */
  bool     extra_row;
  bool     extra_col;
  int      nstep;   /* Number of steps to simulate */
  /* Datatype to simplify communications */
  //MPI_Datatype global_mapping_types[2][2]; /* Map local domains to global domain */
  //MPI_Datatype local_domain_type;  /* Local domain (excluding halo regions) */
  //MPI_Datatype local_column_type;  /* Local column type for boundary swap */
  //MPI_Datatype local_row_type;     /* Local row type for boundary swap */
  /* Data structure to simplify execution */
  char * count;
} local_life;

enum CELL_STATES {
  DEAD        = 0,
  ALIVE       = 100,
  UNDEF       = 75,
  OUTOFBORDER = -1
};

#define DEFAULT_SIZE_X 12
#define DEFAULT_SIZE_Y 12
#define DEFAULT_NSTEP  5

/* Initialize the local regions and set values to UNDEF */
void initialize_life(local_life* life);
/* Read input data related to local_life struct */
void read_input_data(int argc,char** argv,local_life* life);
/* Set initial conditions for life domain */
void set_initial_condition(local_life* life);
/* Swap local boundary */
void swap_local_boundary(local_life* life);
/* Update local domain */
void update_local_domain(local_life* life);
/* Update global domain */
void update_global_domain(local_life* life);
/* Print local domain to stdout for debugging purpose */
void print_local_region(const local_life* life);
/* Print count of alive neighbors for debugging purpose */
void print_local_count(local_life* life);
/* Print domain to stdout for debugging purpose */
void print_to_stdout(const local_life* life);
/* Print to pgm files for visualization */
void print_to_pgm_file(const char* filename, const local_life* life);
/* Free the data structure */
void terminate_life(local_life* life);

#endif
